library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ov7670_capture is
  port (
    pPCLK  : in STD_LOGIC;
    pVSYNC : in STD_LOGIC;
    pHREF  : in STD_LOGIC;
    pDin   : in STD_LOGIC_VECTOR (7 downto 0);

    pAddr        : out STD_LOGIC_VECTOR (9 downto 0);  -- range from 0 to 640
    pDout        : out STD_LOGIC_VECTOR (31 downto 0);
    pWe0         : out STD_LOGIC := '0';
    pWe1         : out STD_LOGIC := '0';
    pLineCounter : out STD_LOGIC_VECTOR(15 downto 0)
    );
end ov7670_capture;

architecture RTL of ov7670_capture is
  
  signal data : std_logic_vector(31 downto 0) := (others => '0');
  signal addr : unsigned(9 downto 0)          := (others => '0');
  
  signal we        : std_logic_vector(3 downto 0)  := (others => '0');
  
  signal vsync_reg : std_logic := '0';
  signal href_reg0 : std_logic := '0';
  signal href_reg1 : std_logic := '0';
  signal data_reg  : std_logic_vector(7 downto 0) := (others => '0');

  signal cLineCounter  : unsigned(15 downto 0) := (others => '0');
  signal cPixelCounter : unsigned(15 downto 0) := (others => '0');
  attribute keep : string;
  attribute keep of cLineCounter : signal is "true";
  attribute keep of cPixelCounter : signal is "true";

  signal first_pixel : std_logic := '0';
  signal first_line : std_logic := '0';

begin

  pAddr <= std_logic_vector(addr);

  pWe0 <= we(3) when cLineCounter(0) = '0' else '0';
  pWe1 <= we(3) when cLineCounter(0) = '1' else '0';

  pDout <= data;
  pLineCounter <= std_logic_vector(cLineCounter);
  
  capture_process : process(pPCLK)
  begin
    if pPCLK'event and pPCLK = '1' then
      data_reg  <= pDin;
      href_reg0 <= pHREF;
      href_reg1 <= href_reg0;
      vsync_reg <= pVSYNC;
      
      if vsync_reg = '1' then
        cLineCounter <= (others => '0');
        we           <= (others => '0');
        first_line   <= '1';
      else
        if href_reg0 = '0' then
          we            <= (others => '0');
          addr          <= (others => '0');
          cPixelCounter <= (others => '0');
        elsif href_reg1 = '0' and href_reg0 = '1' then
          we <= we(3 downto 1) & '1';
          if first_line = '0' then
            cLineCounter <= cLineCounter + 1;
          end if;
          data        <= X"000000" & data_reg;
          first_line  <= '0';
          first_pixel <= '1';
        elsif href_reg0 = '1' then
          first_pixel <= '0';
          data <= data(23 downto 0) & data_reg;
          we   <= we(2 downto 0) & we(3);
          if we(0) = '1' and first_pixel = '0' then
            cPixelCounter <= cPixelCounter + 1;
            addr          <= addr + 1;
          end if;
        end if;
      end if;
    end if;
  end process;
  
end RTL;
