channel uint inCtrlCh  __attribute__((io("aux_ctrl_in")));
channel uint outCtrlCh __attribute__((io("aux_ctrl_out")));
channel uint inDataCh  __attribute__((io("aux_data_in")));
channel uint outDataCh __attribute__((io("aux_data_out")));

#define SCCB_REQ_FLAG      (0x40000000)
#define SCCB_BUSY_FLAG     (0x80000000)
#define SCCB_WE_FLAG       (0x20000000)
#define SCCB_RD_FLAG       (0x00000000)
#define FRAME_CAPTURE_FLAG (0x10000000)

__kernel 
void ov7670_ctrl (
		  int mode, // 0: read_mode, 1: write_mode, 2: capture kick
		  int addr,
		  int din,
		  __global int * restrict dout
		  )
{
  int v = 0, r = 0;
  for(int state = 0; state < 4;){
    write_channel_altera(outCtrlCh, v);
    r = read_channel_altera(inCtrlCh);
    mem_fence(CLK_CHANNEL_MEM_FENCE);
    switch(state){
    case 0:
      v = 0;
      if((r & (SCCB_REQ_FLAG|SCCB_BUSY_FLAG)) == 0){ // deassert REQ_FLAG and check BUSY
	state = 1;
      }
      break;
    case 1: // check status and set value
      v = 0;
      if(mode == 2){ // capture kick
	v = FRAME_CAPTURE_FLAG;
      }else{
	v |= (addr & 0x000000FF);
	v |= ((din & 0x000000FF) << 8);
	v |= (mode == 1) ? SCCB_WE_FLAG : SCCB_RD_FLAG;
	v |= SCCB_REQ_FLAG;
      }
      state = 2;
      break;
    case 2:
      if((r & SCCB_BUSY_FLAG) != 0){ // the operation has been started
	v = 0;
	state = 3;
      }
      break;
    case 3:
      if((r & SCCB_BUSY_FLAG) == 0){ // the operation has been finished
	*dout = (r & 0x00FF0000) >> 16;
	state = 4; // exit
      }
      break;
    default:
      v = 0;
      state = 0;
      break;
    }
  }
}

#define RANGE(x) (x < 16 ? 16 : x > 235 ? 235 : x)

__kernel 
void ov7670_capture (
		     __global unsigned char * restrict buf
		     )
{
  const int size = 320 * 480;
  for(int i = 0; i < size; i++){
    int d = read_channel_altera(inDataCh);
    int y0 = (d >> 24) & 0x000000FF;
    int cb = (d >> 16) & 0x000000FF;
    int y1 = (d >>  8) & 0x000000FF;
    int cr = (d >>  0) & 0x000000FF;
    int r0, g0, b0;
    r0 = y0 + 140750 * (cr - 128) / 100000;
    g0 = y0 -  34550 * (cb - 128) / 100000 - 71690 * (cr - 128) / 100000;
    b0 = y0 + 177900 * (cb - 128) / 100000;
    int r1, g1, b1;
    r1 = y1 + 140750 * (cr - 128) / 100000;
    g1 = y1 -  34550 * (cb - 128) / 100000 - 71690 * (cr - 128) / 100000;
    b1 = y1 + 177900 * (cb - 128) / 100000;
    int ptr;
    ptr = 3*(640*480-1-(2*i));
    buf[ptr+0] = (unsigned char)(RANGE(b0));
    buf[ptr+1] = (unsigned char)(RANGE(g0));
    buf[ptr+2] = (unsigned char)(RANGE(r0));
    ptr = 3*(640*480-1-(2*i+1));
    buf[ptr+0] = (unsigned char)(RANGE(b1));
    buf[ptr+1] = (unsigned char)(RANGE(g1));
    buf[ptr+2] = (unsigned char)(RANGE(r1));
  }
}
